#procedure triown(P,PA,PB,P1,P3)
*
*   Routine solves the triangle recursion
*                     P
*      N1 -------------------------- N3
*          P1    \    N2    /    P3
*                 \        /
*                A \      / B
*                PA \    / PB
*                    \  /
*                     \/
*
*   A,B,N1,N2,N3 are the powers of the denominators
*       We assume that A and B are integers here. Otherwise see triangl2.
*   P is the momentum in N2. We need it here to determine the extra
*   momenta in the numerator.
*
*   We follow the algorithm of F.V.Tkachov Theor. Mat. Fiz. 56(1983)350.
*
*   There are three sums:
*   One in which the power of 1 becomes zero, one for 2 and one for 3.
*   Each sum has 4 constants to sum over, except that for each
*   one of them has its maximal value:
*   k1 = (a-A)-k2
*   k2 = N1-n1          sum1: N1
*   k3 = (b-B)-k4
*   k4 = N3-n3          sum3: N3
*   k1+k3 = N2-n2       sum2: N2
*   The gamma functions are evaluated using the tables in pochtabl.prc
*
*   It turns out that the easy cases are faster when the regular recursion
*   is used. We do that here is there are fewer than 6 powers in P,P1 and P3
*   combined. Test timings don't use these special cases.
*
*   Programmed by J.A.M.Vermaseren 28-oct-1990 + 13-nov-1990
*
*   Test versus benzbar:
*       1/p1.p1^3/p.p/p3.p3/pa.pa/pb.pb -> 0.96 sec versus 0.80 sec
*       1/p1.p1^3/p.p^3/p3.p3/pa.pa/pb.pb -> 3.07 sec versus 20.34 sec
*       1/p1.p1^3/p.p^3/p3.p3^3/pa.pa/pb.pb -> 6.21 sec versus 547.11 sec
*   triangle generates exactly the right number of terms
*   (times on Atari TT)
*
*   For the easy cases we take the original recursion, because this is much
*   faster. This is done for the combined powers of P1,P2,P3 at most 4
*   in the denominator. IT IS VERY DANGEROUS TO INCREASE THIS NUMBER!
*   In principle each two steps in the recursion can generate one
*   extra pole that has to be cancelled between the terms. This can
*   cause problems with the truncation of the powers of ep. We can
*   tolerate only one such pole here.
*
id  'P' = x*'P';
if ( count('P1'.'P1',1,'P3'.'P3',1,'P'.'P',1) < -4 );
id  x^m?/'P1'.'P1'^N1?/'P'.'P'^N2?/'P3'.'P3'^N3?/'PA'.'PA'^A?/'PB'.'PB'^B? =
        ftri(N1,N2,N3,A,B,m)/fac_(A-1)/fac_(B-1);
id  ftri(N1?,N2?,N3?,A?,B?,m?) =
        +sum_(k4,0,N3-1,(-1)^(N1+k4)*po(5+m-2*N2-A-B-N1-k4,-2)
            *sum_(k1,0,N2-1,sum_(k3,0,N2-k1-1,
            fac_(k1+N1+k3+k4-1)/fac_(k3)
            *poinv(5+m-2*N2-A-B+k1+k3,-2)
            *ftri(0,N2-k1-k3,N3-k4,A+k1+N1,B+k3+k4)
         )/fac_(k1))/fac_(k4))/fac_(N1-1)

        +sum_(k2,0,N1-1,sum_(k4,0,N3-1,(-1)^(k2+k4)*fac_(k2+N2+k4-1)
            *po(4+m-2*N2-A-B-k2-k4,-2)
            *sum_(k1,0,N2,1/fac_(k1)/fac_(N2-k1)
            *ftri(N1-k2,0,N3-k4,A+k1+k2,B+N2-k1+k4)
         )/fac_(k2)/fac_(k4)))*N2*poinv(4+m-2*N2-A-B+N2,-2)

        +sum_(k2,0,N1-1,(-1)^(k2+N3)*po(5+m-2*N2-A-B-k2-N3,-2)
            *sum_(k3,0,N2-1,sum_(k1,0,N2-k3-1,
            fac_(k1+k2+k3+N3-1)/fac_(k1)
            *poinv(5+m-2*N2-A-B+k1+k3,-2)
            *ftri(N1-k2,N2-k1-k3,0,A+k1+k2,B+k3+N3)
         )/fac_(k3))/fac_(k2))/fac_(N3-1)
    ;
id  ftri(N1?,N2?,N3?,A?,B?) = fac_(A-1)*fac_(B-1)
    /'P1'.'P1'^N1/'P'.'P'^N2/'P3'.'P3'^N3/'PA'.'PA'^A/'PB'.'PB'^B;
*
*   Evaluate the gamma functions.
*   We multiply numerator and denominator with gamma(1-2*ep)
*   Then po(x,-2) represents a normalized gamma function.
*   The following identities can then be applied:
*   (the table has overal factors ep taken out)
*
id  po(1,-2) = 1;
id  poinv(1,-2) = 1;

***id  po(x1?neg0_,-2) = -acc(PO(x1,-2))/2/ep;
***id  po(x1?,-2) = acc(PO(x1,-2));
***id  poinv(x1?neg0_,-2) = -2*acc(POINV(x1,-2))*ep;
***id  poinv(x1?,-2) = acc(POINV(x1,-2));

else;

  repeat;
    if ( match(1/'P'.'P'/'P1'.'P1'/'P3'.'P3') > 0 );
      id  x^x4?/'PA'.'PA'^x1?/'PB'.'PB'^x2?/'P'.'P'^x3? =
          x^x4 /'PA'.'PA'^x1 /'PB'.'PB'^x2 /'P'.'P'^x3 *(
            +x2*'P'.'P'/'PB'.'PB'-x2*'P3'.'P3'/'PB'.'PB'
            +x1*'P'.'P'/'PA'.'PA'-x1*'P1'.'P1'/'PA'.'PA'
          )*deno(4+x4-x1-x2-2*x3,-2);
***      id,many,deno(0,-2)   = -1/2/ep;
***      id,many,deno(x1?,-2) = accm(1-2*ep/x1)/x1;
    endif;
  endrepeat;
***  repeat id accm(x1?)*accm(x2?) = accm(x1*x2);
***  id  accm(x1?) = accm(x1-1);
***  id  accm(x1?) = accm(x1-x1^2,x1^3);
***  id  accm(x1?,x2?) = acc(1-x1-x2+x2*x1+x2^2);
  id  x = 1;

endif;

repeat id acc(x1?)*acc(x2?) = acc(x1*x2);

#endprocedure
